Unit Fft;

interface

uses Maths, sysUtils;

const
    fenetreHamming : boolean = false;
    maxVecteurFFT = 2048;
    maxFFT = 1024;

Procedure Fourier(Nbre : integer;Duree : double;var X,F,Amplitude : vecteur);
Procedure InverserIndices;

Implementation

Function PuissDeux(N:integer) : integer; { 2^N avec N>=0 }
var Y,i : integer;
begin
    Y := 1;
    for i := 1 to N do Y := Y*2;
    PuissDeux := Y;
end;

var tbi : vecteurInt;
    wr,wi,kf : vecteur;

PROCEDURE inverserIndices;
{ Table de l'ordre inverse des bits }
var i,j,k,r,p0,q,tiroir : integer;
    dpsn : double;
begin
  dpsn:=2*pi/MaxVecteurFFT;
  FOR i:=0 TO pred(MaxVecteurFFT) do begin
         wr[i]:=COS(dpsn*i);
         wi[i]:=SIN(-dpsn*i);
         if fenetreHamming
{ ATTENTION : les indices ne sont pas permuts }
{ la permutation selon tbi[] sera faite dans le fentrage }
            then kf[i]:=0.54-0.46*cos(i*dpsn)
            else kf[i]:=1.0;
    end;
    FOR i:=0 TO pred(MaxVecteurFFT) do tbi[i]:=i;
    r:=trunc(0.5+Ln(MaxVecteurFFT)/ln(2));
    FOR i:=0 TO pred(MaxVecteurFFT) do begin
      p0:=r;
      q:=1;
      k:=0;
      FOR j:=1 TO r do begin
           IF (i AND puissDeux(PRED(p0)))<>0 then k:=k+q;
           q:=2*q;
           DEC(p0);
      end;
      IF k>i then begin
            tiroir:=tbi[k];
            tbi[k]:=tbi[i];
            tbi[i]:=tiroir;
      end;
    end;
end; {inverserIndices}

Procedure Fourier(Nbre : integer;Duree : double;var X,F,Amplitude : vecteur);
var xr,xi : vecteur;

PROCEDURE tfd;
 { Transformation proprement dite }
var a,i,j,k,k0,nbbloc,tbloc,r,r0,p0,nombre0 : integer;
    zr,zi,c : double;
begin
  nbbloc:=maxVecteurFFT;
  tbloc:=1;
  r:=trunc(0.5+Ln(MaxVecteurFFT)/ln(2));
  FOR i:=1 TO r do begin
          tbloc:=2*tbloc;
          nbbloc:=nbbloc div 2;
          FOR j:=1 TO nbbloc do begin
             nombre0:=(MaxVecteurFFT div nbbloc -1) div 2;
             r0:=0;
             FOR k0:=0 TO nombre0 do begin
                    k:=k0+(pred(j))*tbloc;
                    p0:=k+tbloc div 2;
                    a:=r0;
                    zr:=xr[p0]*wr[a]-xi[p0]*wi[a];
                    zi:=xr[p0]*wi[a]+xi[p0]*wr[a];
                    xr[p0]:=xr[k]-zr;
                    xi[p0]:=xi[k]-zi;
                    xr[k]:=xr[k]+zr;
                    xi[k]:=xi[k]+zi;
                    r0:=r0+nbbloc;
             end; {k0}
          end; {j}
  end; {i}
  c := 2/MaxVecteurFFT;
  for i := 0 to pred(MaxVecteurFFT) do begin
       xr[i]:=c*xr[i];
       xi[i]:=c*xi[i];
  end;
  xr[0] := 2*xr[0];
  xi[0] := 2*xi[0];
end; // tfd

procedure fenetrerHamming;
const coeff = 1.85;
var j : integer;
begin
     for j:=0 to pred(MaxVecteurFFT) do begin
         xr[tbi[j]]:=kf[j]*X[j];
// rangement avec permutation des mesures tableau fentres dans xr[]
         xi[j]:=0;
     end;
     for j:=0 to pred(maxVecteurFFT) do { renormaliser }
         xr[j]:=xr[j]*coeff;
end; // Hamming

procedure fenetrerRect;
var j : integer;
begin
     for j:=0 to pred(MaxVecteurFFT) do begin
         xr[tbi[j]]:=x[j];
{ rangement avec permutation des mesures tableau fentres dans xr[] }
         xi[j]:=0;
     end;
end; // rectangle

var i : integer;
    Df,coeff : double;
    periode : double;
begin
     Periode := duree/pred(Nbre)*Nbre;
     while nbre>maxVecteurFFT do begin
         nbre := nbre div 2;
         for i := 0 to pred(Nbre) do X[i] := X[2*i];         
     end;
     for i := Nbre to pred(MaxVecteurFFT) do X[i] := 0;
     if fenetreHamming
        then fenetrerHamming
        else fenetrerRect;
     tfd;
     Df := 1/periode;
     Df := Df*Nbre/MaxVecteurFFT;
     coeff := MaxVecteurFFT/Nbre;
     for i := 0 to pred(MaxFFT) do begin
         xr[i] := xr[i]*coeff;
         xi[i] := xi[i]*coeff;
         F[i] := i*Df;
         amplitude[i]:=sqrt(sqr(xr[i])+sqr(xi[i]));
     end;
end;

initialization
    InverserIndices

end.

