unit GTIDef;

interface

uses classes, sysutils, oomisc, inifiles, graphics, StdCtrls, forms,
     uniteker, maths, math, gtiutil, adport, constgti, registry, parser10;

Const
    premiereFois : boolean = true;
    voieT = 1;
    voieMagnum = 2;
    voieSA = 3;
    voieClavier = 4;
    voieF = 5;
    voieA = 6;voieB = 7;voieC = 8;voieD = 9;voieE =10;
    voieAnaMax = 10;
    voieMin = voieT;
    voieEB = 11;
    voieSB = 12;
    voieNum = 0;
    VoieMax = voieSB;
    MaxMoyenne = 17;
    MaxiHardSA = 255;
    Hard5V = 32767;
    Hard5VNeg = 32768;
    MaxiFrequence = 1E6;
    MiniFrequence = 10;
    MinPoints = 4;
    DeltaHorloge = 0.27126E-6;
    VoieOrphyEF0 = 16;
    MaxResulGTI = 32768;
    NbreMaxSA = 2048;
    MaxiFreqGP305 = 5e4;
    MiniFreqGP305 = 0.05;
    DeltatLent = 0.055;
    indexMarteau = 1;

    OrdreWMpotar = 'YUTIL 10 I 41472';
    OrdreWRpotar = 'YUTIL 11 I 41728';
    OrdreWEpotar = 'YUTIL 9 I 41216';
    OrdreWRECO = 'YUTIL 12 I 41984';

type
    Tmotif = (mCroix,mCroixDiag,mCercle,mCarre,mLosange);
    TCalibreAmpli = (ca_05,ca_1,ca_10,ca_100,ca_1000,ca_cal);
    TModeAcq = (AcqClavier,AcqTemps,AcqPoint,AcqMagnum,AcqGP305,AcqSA);
    TetatAcq = (eaAttenteSynchro,eaAttenteReveil,eaAttenteResul,eaMajResul,
       eaAttenteResulMag,eaTemporelLent,eaAttenteStopMag,eaFlush,eaWait,eaTerminee);
    TmodeTemporel = (mNormal,mRapide,mLent);
    TGenreCapteur = (gNone,gEntree,gSortie,gPhase,gPhotocolor);
    indiceVoie = voieNum..VoieMax;
    indiceVoieAna = voieA..VoieE;
    TindexMinMax = 0..6;
    TModeSynchro = (S_front,S_seuil,S_relaxe,S_clavier);
    EserieError= class(Exception);
    EgtiError= class(Exception);

Const
    GainAmpli : array[TcalibreAmpli] of double = (0.5,1,10,100,1000,1);
    codeMontant : array[boolean] of char = ('D','M');
    codeFront : array[0..3] of char = ('0','1','2','3');
    DeltatMax = 3600.0; { s = 1 heure }
    DureeMaxMagnum = 6.5536*6.5536*10;
    DureeMax = deltatMax*MaxVecteur;
      { 8192 heures = 341 jours = 11 mois }
      { 1024 heures = 43 jours = 1,5 mois }
    DeltatOrphy = 1.0e-5;
    DeltatOrphyRap = 3.2e-6;
    VitesseChar : array[0..2] of Char = ('4','5','6');  { 4] '7','8' }
    VitesseInt : array[0..2] of LongInt = (9600,19200,38400); { 57600, 115200 }
    PriseVoie : array[0..15] of char = 'AADDBBCCGHIJGHIJ';
    LongueurUnite = 15;
    signifNumero : array[0..15] of string =
        ('EA0 prise A',
         'EA1 prise A',
         'EA2 prise D ou F',
         'EA3 prise D ou F',
         'EA4 prise B',
         'EA5 prise B',
         'EA6 prise C',
         'EA7 prise C',
         'EA8 prise G',
         'EA9 prise H',
         'EA10 prise I',
         'EA11 prise J',
         'EAD0 prise G',
         'EAD1 prise H',
         'EAD2 prise I',
         'EAD3 prise J');
    MaxPotarAuto = 4;
    MaxCapteurAuto = 22;

type
   TcalibreGTI = (c_10,c_5,c_2,c_1,c_500m,c_200m,c_100m,c_5AC);
   indexPotarAuto = 0..MaxPotarAuto;
   Tetalonnage = (eNone,eLin,eNonLin,eAuto);
   chaineUniteCapteur = string[8];
   TTension = (NegAndPos,Positive,Negative);

   TCapteur = class
        AvecPotarC : boolean;
        AmpliPhysiqueC : boolean;
        AlimentationC : boolean;
        Val0Vc,Val5Vc : double;
        nomC : string;
        nomUniteC : string;
        signifC,aideC : string;
        calibreC : TcalibreGTI;
        numeroC : integer;
        TensionC : Ttension;
        LinearC : boolean;
        ExpressionC : string;
        MiniAff,MaxiAff : double; /// ????
        Genre : TGenreCapteur;
        constructor create;
        procedure Assign(Acapteur : Tcapteur);
    end;

       TCapteurAuto = class
        private
             FCalibre : TcalibreGTI;
             procedure setCalibre(Acalibre : TcalibreGTI);
        public
        PotarActif : array[indexPotarAuto] of boolean;
        ValeurPotar : array[indexPotarAuto] of word;
        ValeurAttendue : array[indexPotarAuto] of double;
        AidePotar : array[indexPotarAuto] of String;
        OffsetC : double;
        UnVoltC : double;
        nomC : string;
        nomUniteC : string;
        signifC : String;
        MiniAff,MaxiAff : double;
        PenteV,DecalageV : double;
        EtalonnageFait : boolean;
        TensionC : TTension;
        capteurSynchro : boolean;
        isMagnum : boolean;
        property calibre : TcalibreGTI read Fcalibre write setCalibre;
        constructor create;
        function ValeurReal(V : integer) : double;
        function AvecEtalonnage : boolean;
   end;

   Tvoie = class(Tunite)
          private
          Fpente,FpenteV,FdecalageV,Fdecalage : double;
          function getExpression : string;
          procedure setExpression(Avalue : string);
          procedure setPente(Avalue : double);
          protected
          Parser : Tparser;
          VoltParser : PParserFloat;
          public
          oldValeurs : array[0..maxMoyenne] of double;
          NbreMoyenne : integer;
          exponent : integer;
          Calibre : TcalibreGTI;
          Valeur0V,Valeur5V : double;
          NumeroOrphy : integer;
          ValeurCourante,valeurAff : double;
          Tension : TTension;
          Inverse,AvecPotar,AmpliPhysique,Alimentation : boolean;
          signif,aide : String;
          Mini,Maxi,MiniAffNL,MaxiAffNL,MaxiFFT : double;
          exposantAff : double;
          nbreAvantVirgule : integer;
          vecteurAff,vecteurFFT : vecteur;
          vecteurCourant : vecteurInt;
          active : boolean;
          couleur : Tcolor;
          Motif : Tmotif;
          Etalonnage : Tetalonnage;
          LabelVoie : Tlabel;
          GroupBox : TgroupBox;
          indexCapteurAuto : integer;
          isPhotocolor : boolean;
          constructor create(num : integer);
          procedure Assign(source : Tpersistent);override;
          Function CodeVoie : string;
          Function NomVoieOrphy : string;
          Function CalibreVoie : string;virtual;
          Function CommandeCalibre : shortString;
          Procedure ChercheCapteur(genre : TgenreCapteur);
          Procedure AffecteCapteur(capteurCourant : Tcapteur);
          Function AffVoltmetre : string;virtual;
          procedure CalculPente;
          Function ValeurInt(V : double) : word;
          Function ValeurReal(V : integer) : double;virtual;
          Function ValeurVolt(V : integer) : double;virtual;
          procedure RazVecteur;
          procedure load(avoie : string;Fichier : tMemIniFile);virtual;
          procedure store(Avoie : string;Fichier : tMemIniFile);virtual;
          destructor Destroy;override;
          property expression : string read getExpression write setExpression;
          property pente : double write setPente;
    end;

const
   crLF = #10;
   crCR = #13;
   GainCalibre : array[TcalibreGTI] of double = (0.5,1,2.5,5,10,25,50,1);
   ValeurCalibre : array[TcalibreGTI] of double = (10,5,2,1,0.5,0.2,0.1,5);
   nomPotar : array[indexPotarAuto] of string =
      ('offset zoom','gain zoom','offset','gain','bonus');

type
  TlisteCapteur = class(TList)
  protected
    function Get(Index : Integer) : Tcapteur;
    procedure Put(Index : Integer;Item : Tcapteur);
  public
    function Add(Item : Tcapteur) : Integer;
    function IndexOf(Item : Tcapteur) : Integer;
    function Remove(Item : Tcapteur) : Integer;
    property Items[Index : Integer] : Tcapteur read Get write Put;
    Procedure Clear;override;
  end;


var
    NomFichierCapteurs : String;
    ListeCapteur : TlisteCapteur;
    ListeCapteursAuto : array[1..MaxCapteurAuto] of TCapteurAuto;

Procedure ChargeListeCapteurs;
Procedure SauveListeCapteurs;
Procedure GetCodeGP305(f : double;var iSB,iSA : integer);

function nomEtatAcq(etat : TetatAcq) : String;
function nomModeAcq(mode : TModeAcq) : String;
function nomSynchro(mode : TModeSynchro) : String;

implementation

const
    CouleurDefaut : array[indiceVoie] of Tcolor =
       (clBlack,clBlack,clRed,clBlack,clRed,clNavy,clGreen,clPurple,clBlue,
        clFuchsia,clBlack,clBlack,clBlack);

Procedure SauveListeCapteurs;
{  sauvegarde de la liste  }
var fichier : tMemIniFile;
    ligne : String;
    i,j : integer;
    regIni : TregIniFile;
begin
    fichier := tMemIniFile.create(nomFichierCapteurs);
    try
    fichier.writeInteger('Capteurs','Nombre',ListeCapteur.count);
    for i := 0 to pred(ListeCapteur.count) do
       with ListeCapteur.items[i] do begin
        ligne := IntToStr(i+1);
        fichier.writeBool(ligne,'Etalonnage',AvecPotarC);
        fichier.writeBool(ligne,'Ampli',AmpliPhysiqueC);
        fichier.writeBool(ligne,'Alim',AlimentationC);
        fichier.writeBool(ligne,'Linear',LinearC);
        fichier.writeFloat(ligne,'0V',Val0Vc);
        fichier.writeFloat(ligne,'5V',Val5Vc);
        fichier.writeString(ligne,'Nom',nomC);
        fichier.writeString(ligne,'Signif',signifC);
        if linearC
           then fichier.writeString(ligne,'Expression','')
           else fichier.writeString(ligne,'Expression',ExpressionC);
        fichier.writeInteger(ligne,'Calibre',ord(calibreC));
        fichier.writeInteger(ligne,'Numero',numeroC);
        fichier.writeString(ligne,'Aide',aideC);
        fichier.writeInteger(ligne,'Tension',ord(TensionC));
    end;
    for i := 1 to MaxCapteurAuto do
       with ListeCapteursAuto[i] do begin
            ligne := 'Auto'+IntToStr(i);
            for j := 0 to MaxPotarAuto do begin
                fichier.writeInteger(ligne,'ValeurPotar'+intToStr(j),valeurPotar[j]);
                fichier.writeBool(ligne,'PotarActif'+intToStr(j),PotarActif[j]);
                fichier.writeString(ligne,'AidePotar'+intToStr(j),aidePotar[j]);
                fichier.writeFloat(ligne,'Valeur'+intToStr(j),valeurAttendue[j]);
            end;
            fichier.writeFloat(ligne,'0V',Offsetc);
            fichier.writeInteger(ligne,'Signe',ord(tensionC));
            fichier.writeFloat(ligne,'1V',UnVoltc);
            fichier.writeInteger(ligne,'Calibre',ord(Fcalibre));
            fichier.writeFloat(ligne,'Mini',MiniAff);
            fichier.writeFloat(ligne,'Maxi',MaxiAff);
            fichier.writeString(ligne,'Nom',nomC);
            fichier.writeString(ligne,'Unite',nomUniteC);
            fichier.writeString(ligne,'Signif',signifC);
            fichier.writeBool(ligne,'Synchro',capteurSynchro);
            fichier.writeBool(ligne,'Magnum',isMagnum);
       end;
    fichier.UpdateFile;
    fichier.free;
    except
    end;
    try
    RegIni := TRegIniFile.create(RegistreGTI);
    RegIni.writeString('Capteurs','Fichier',NomFichierCapteurs);
    RegIni.free;
    except
    end;
end;

Procedure ChargeListeCapteurs;
var fichier : tMemIniFile;

Procedure CapteursAuto;
var ligne : String;
    i,j : integer;
begin
    for i := 1 to MaxCapteurAuto do begin
      ListeCapteursAuto[i] := TcapteurAuto.create;
      with ListeCapteursAuto[i] do begin
        ligne := 'Auto'+IntToStr(i);
        EtalonnageFait := true;
        for j := 0 to maxPotarAuto do begin
            valeurPotar[j] := fichier.readInteger(ligne,'ValeurPotar'+intToStr(j),0);
            PotarActif[j] := fichier.readBool(ligne,'PotarActif'+intToStr(j),false);
            if PotarActif[j] then EtalonnageFait := false;
            aidePotar[j] := fichier.readString(ligne,'AidePotar'+intToStr(j),'');
            valeurAttendue[j] := fichier.readFloat(ligne,'Valeur'+intToStr(j),0);
        end;
        OffsetC := fichier.ReadFloat(ligne,'0V',0);
        TensionC := Ttension(fichier.ReadInteger(ligne,'Signe',0));
        UnVoltC := fichier.ReadFloat(ligne,'1V',1);
        MiniAff := fichier.ReadFloat(ligne,'Mini',0);
        MaxiAff := fichier.ReadFloat(ligne,'Maxi',5);
        nomC := fichier.readString(ligne,'Nom',nomC);
        calibre := TcalibreGTI(fichier.readInteger(ligne,'Calibre',0));
        nomUniteC := fichier.readString(ligne,'Unite','');
        signifC := fichier.readString(ligne,'Signif','');
        capteurSynchro := fichier.readBool(ligne,'Synchro',false);
        isMagnum := fichier.readBool(ligne,'Magnum',false);
    end end;
end; { CapteursAuto }

{  chargement de la liste si vide cration  l'aide de defaut }
var ligne : String;
    i : integer;
    NbreCapteurs : integer;
    CapteurCourant : Tcapteur;
begin
    fichier := tMemIniFile.create(nomFichierCapteurs);
    listeCapteur.clear;
    NbreCapteurs := fichier.readInteger('Capteurs','Nombre',0);
    for i := 1 to NbreCapteurs do begin
      CapteurCourant := Tcapteur.create;
      ListeCapteur.add(CapteurCourant);
      with CapteurCourant do begin
        ligne := IntToStr(i);
        AvecPotarC := fichier.readBool(ligne,'Etalonnage',false);
        LinearC := fichier.readBool(ligne,'Linear',true);
        AmpliPhysiqueC := fichier.readBool(ligne,'Ampli',false);
        AlimentationC := fichier.readBool(ligne,'Alim',false);
        Val0Vc := fichier.readFloat(ligne,'0V',0);
        Val5Vc := fichier.readFloat(ligne,'5V',5);
        if linearC
           then expressionC := ''
           else begin
              expressionC := fichier.readString(ligne,'Expression','');
              linearC := expressionC='';
           end;
        nomC := fichier.readString(ligne,'Nom',nomC);
        nomUniteC := fichier.readString(ligne,'Unite','');
        signifC := fichier.readString(ligne,'Signif','');
        calibreC := TcalibreGTI(fichier.readInteger(ligne,'Calibre',0));
        aideC := fichier.readString(ligne,'Aide','');
        numeroC := fichier.readInteger(ligne,'Numero',numeroC);
        if nomC='' then nomC := 'V'+intToStr(numeroC);
        TensionC := Ttension(fichier.readInteger(ligne,'Tension',0));
        genre := TgenreCapteur(fichier.ReadInteger(ligne,'Genre',0));        
    end end;
    if NbreCapteurs=0 then begin
       CapteurCourant := TCapteur.create;
       ListeCapteur.add(CapteurCourant);
    end;
    CapteursAuto;
    fichier.free;
end; { ChargeListeCapteurs }

Procedure Tvoie.Store(Avoie : string;Fichier : tMemIniFile);
begin with fichier do begin
      WriteString(Avoie,'Nom',nom);
      WriteString(Avoie,'Unite',nomUnite);
      WriteBool(Avoie,'Etalonnage',avecPotar);
      WriteInteger(Avoie,'Capteur',ord(Etalonnage));
      WriteString(Avoie,'Expression',Expression);
      WriteString(Avoie,'Signif',signif);
      WriteString(Avoie,'Aide',aide);      
      WriteFloat(Avoie,'Mini',mini);
      WriteFloat(Avoie,'Maxi',maxi);
      WriteInteger(Avoie,'Moyenne',NbreMoyenne);
      WriteInteger(Avoie,'Calibre',ord(calibre));
      WriteFloat(Avoie,'Valeur0V',valeur0V);
      WriteFloat(Avoie,'Valeur5V',valeur5V);
      WriteFloat(Avoie,'MiniAff',miniAffNL);
      WriteFloat(Avoie,'MaxiAff',maxiAffNL);
      WriteInteger(Avoie,'NumeroOrphy',numeroOrphy);
      WriteInteger(Avoie,'Tension',ord(tension));
      WriteBool(Avoie,'Active',active);
      WriteInteger(Avoie,'Couleur',couleur);
      WriteInteger(Avoie,'Motif',ord(motif));
      WriteBool(Avoie,'Photocolor',IsPhotocolor);
end end; // Tvoie.store

Destructor Tvoie.destroy;
begin
     parser.free
end;

procedure Tvoie.setPente(Avalue : double);
begin
    if etalonnage=eAuto then Fpente := avalue
end;

Constructor Tvoie.create(num : integer);
var i : integer;
begin
     inherited create;
     nom := 'V'+intToStr(num);
     nomUnite := 'V';
     signif := '';
     aide := '';
     Mini := -5;
     Maxi := 5;
     MiniAffNL := 0;
     MaxiAffNL := 25;
     Calibre := c_5;
     Valeur0V := 0;
     Valeur5V := 5;
     Inverse := false;
     Tension := NegAndPos;
     ValeurCourante := 0;
     numeroOrphy := num-voieA;
     indexCapteurAuto := 0;
     AmpliPhysique := false;
     Alimentation := false;
     avecPotar := false;
     Motif := mCroix;
     etalonnage := eNone;
     parser := nil;
     couleur := CouleurDefaut[num];
     exposantAff := 1;
     exponent := 0;
     NbreAvantVirgule := 0;
     for i := 0 to maxMoyenne do
         oldValeurs[i] := sansSignif;
end;

Procedure Tvoie.CalculPente;

Procedure Reset;
begin
        Fpente := 1;
        Fdecalage := 0;
        valeur0V := 0;
        valeur5V := 5;
        mini := valeurVolt(Hard5Vneg);
        maxi := valeurVolt(Hard5V);
        ExposantAff := 1;
        Exponent := 0;
        NbreAvantVirgule := floor(Log10(5))+2;
        case Tension of
              NegAndPos : ;
              Positive : mini := 0;
              Negative : maxi := 0;
        end;
end;

var maxmax : double;
begin
   FpenteV := 5/Hard5V/GainCalibre[calibre];
   FdecalageV := -FpenteV*65535.0;
   case etalonnage of
      eAuto : with ListeCapteursAuto[indexCapteurAuto] do begin
           Fpente := unVoltC;
           Fdecalage := offsetC;
           valeur0V := offsetC;
           valeur5V := offsetC+5*unVoltC;
           mini := miniAff;
           maxi := maxiAff;
      end;
      eLin : begin
        Fpente := (valeur5V-valeur0V)/Hard5V/GainCalibre[calibre];
        Fdecalage := -Fpente*65535.0;
        Inverse := Fpente<0;
        if Inverse
           then begin
              case Tension of
                 NegAndPos : BEGIN
                    maxi := valeurReal(Hard5V);
                    mini := valeurReal(Hard5VNeg);
                 END;
                 Positive : begin
                    maxi := valeur0V;
                    mini := valeurReal(Hard5V);
                 end;
                 Negative : begin
                    mini := valeur0V;
                    maxi := valeurReal(Hard5VNeg);
                 end;
              end;
          end
          else begin
              case Tension of
                 NegAndPos : begin
                  mini := valeurReal(Hard5Vneg);
              maxi := valeurReal(Hard5V);
                 end;
                 Positive : begin
                    mini := valeur0V;
                    maxi := valeurReal(Hard5V);
                 end;
                 Negative : begin
                   maxi := valeur0V;
                   mini := valeurReal(Hard5Vneg);
                 end;
              end;
          end;
      end;
      eNonLin : begin { non linaire }
          maxi := maxiAffNL;
          mini := miniAffNL;
          if maxi=sansSignif then maxi := 0;
          if mini=sansSignif then mini := 0;
          inverse := maxi<mini;
          if inverse then begin
               maxmax := mini;
               mini := maxi;
               maxi := maxmax;
          end;
          if (maxi-mini)<1e-12 then begin
             reset;
             afficheErreur('Expression d''talonnage incorrecte',25);
          end;
      end;
      eNone : reset;
   end;
   if valeur0V=valeur5V then reset;
   if (maxi>0) and (abs(mini)<maxi*1e-3) then mini := 0;
   MaxMax := abs(maxi);
   if MaxMax<abs(mini) then MaxMax := abs(mini);
   try
   Exponent := floor(Log10(MaxMax));
   if (Exponent mod 3) < 0
      then Exponent := (Exponent div 3)-1
      else Exponent := Exponent div 3;
  { la division euclidienne des informaticiens <> div des maths ! }
   Exponent := 3*Exponent;
   ExposantAff := Dix(Exponent);
   NbreAvantVirgule := floor(Log10(MaxMax/ExposantAff))+2;
   except
   reset;
   end;
end; // calculPente

function Tvoie.CodeVoie : string;
begin
     codeVoie := IntToStr(numeroOrphy)
end;

Function Tvoie.NomVoieOrphy : string;
begin
     case numeroOrphy of
          0..11 : nomVoieOrphy := 'EA'+intToStr(numeroOrphy);
          12..15 : nomVoieOrphy := 'EAD'+intToStr(numeroOrphy-12);
          else nomVoieOrphy := ''
     end;
end;

Function Tvoie.CommandeCalibre : shortString;
begin
     result := 'YCAL '+codeVoie+' '+chr(ord('0')+ord(calibre))
end;

Function Tvoie.ValeurReal(V : integer) : double;
begin
    case etalonnage of
          eNonLin : if parser<>nil
             then begin
                VoltParser^ := valeurVolt(V);
                try
                result := Parser.Value;
                except
					result := sansSignif;
                end;
             end
             else result := sansSignif;
          eLin : begin
              result := Valeur0V+Fpente*V;
              if V>Hard5V then result := result+Fdecalage;
          end;
          eAuto : begin
              result := valeurVolt(V);
              result := Fdecalage+Fpente*result;
              if V>Hard5V then result := result+Fdecalage;
              if result<mini then result := mini;
              if result>maxi then result := maxi;
          end;
          else result := valeurVolt(V);
      end;
      if abs(result)<(abs(maxi-mini)/Hard5V) then result := 0;
end;

Function Tvoie.ValeurVolt(V : integer) : double;
begin
     result := FpenteV*V;
     if V>Hard5V then result := result+FdecalageV
end;

Function Tvoie.ValeurInt(V : double) : word;
var Vi : longInt;
begin
      Vi := round((V-valeur0V)/Fpente);
      if Vi<0 then Vi := Vi+65535;
      if Vi>65535 then Vi := 65535;
      if Vi<0 then Vi := 0;
      ValeurInt := Vi;
end;

procedure Tvoie.RazVecteur;
var i : integer;
begin
    for i := 0 to MaxVecteur do begin
       vecteurAff[i] := 0;
       vecteurCourant[i] := 0;
       vecteurFFT[i] := 0;
    end;
end;

Procedure Tvoie.Load(Avoie : string;fichier : tMemIniFile);
begin with fichier do begin
      Nom := ReadString(Avoie,'Nom',Avoie);
      NomUnite := ReadString(Avoie,'Unite',NomUnite);
      Signif := ReadString(Avoie,'Signif','');
      Aide := ReadString(Avoie,'Aide','');
      avecPotar := ReadBool(Avoie,'Etalonnage',false);
      etalonnage := Tetalonnage(ReadInteger(Avoie,'Capteur',0));
      if etalonnage=eNonLin
         then begin
            expression := ReadString(Avoie,'Expression','');
            if expression='' then etalonnage := eNone;
         end
         else expression := '';
      Mini := ReadFloat(Avoie,'Mini',-5);
      Maxi := ReadFloat(Avoie,'Maxi',5);
      Calibre := TcalibreGTI(ReadInteger(Avoie,'Calibre',0));
      valeur0V := ReadFloat(Avoie,'Valeur0V',0);
      valeur5V := ReadFloat(Avoie,'Valeur5V',5);
      if (etalonnage=eNone) and
         ((valeur0V<>0) or
          (valeur5V<>5)) then etalonnage := eLin;
      active := ReadBool(Avoie,'Active',false);
      numeroOrphy := ReadInteger(Avoie,'NumeroOrphy',0);
      Couleur := readInteger(Avoie,'Couleur',couleur);
      Motif := Tmotif(readInteger(Avoie,'Motif',0));
      Tension := Ttension(ReadInteger(Avoie,'Tension',0));
      isPhotocolor := ReadBool(Avoie,'Photocolor',false);
      NbreMoyenne := readInteger(Avoie,'Moyenne',1);
      miniAffNL := ReadFloat(Avoie,'MiniAff',0);
      maxiAffNL := ReadFloat(Avoie,'MaxiAff',25);
      if nom='' then nom := 'V'+intToStr(numeroOrphy);
end end;

Function Tvoie.CalibreVoie : string;
begin
   case Tension of
      NegAndPos : if abs(maxi)>abs(mini)
            then result := ''+formatUnite(abs(maxi))
            else result := ''+formatUnite(abs(mini));
      Positive : result := formatUnite(maxi);
      Negative : result := formatUnite(mini);
   end;
end;

Function Tvoie.AffVoltmetre : string;
var posVirgule,i : integer;
begin
    result := FloatToStrF(valeurAff/ExposantAff,ffFixed,3,5-NbreAvantVirgule);
    posVirgule := pos('.',result);
    for i := 0 to (NbreAvantVirgule-posVirgule) do
        result := ' '+result;
end;

constructor TcapteurAuto.create;
var i : integer;
begin
        Offsetc := 0;
        unVoltc := 10;
        nomC := 'T';
        NomUniteC := 'C';
        signifC := '';
        for i := 0 to MaxPotarAuto do begin
            valeurPotar[i] := 0;
            PotarActif[i] := false;
            valeurAttendue[i] := 0;
        end;
end;

Function TcapteurAuto.ValeurReal(V : integer) : double;
begin
     result := penteV*V;
     if V>Hard5V then result := result+decalageV;
     result := OffsetC+unVoltC*result;
end;

function TcapteurAuto.avecEtalonnage;
var i : integer;
begin
     result := false;
     for i := 0 to maxPotarAuto do
         result := result or potarActif[i]
end;

procedure TcapteurAuto.setCalibre(Acalibre : TcalibreGTI);
begin
     Fcalibre := Acalibre;
     penteV := 5/Hard5V/GainCalibre[calibre];
     decalageV := -penteV*65535.0;
end;

function TlisteCapteur.Add(Item : Tcapteur) : Integer;
begin
     result := inherited Add(Item)
end;

function TlisteCapteur.Get(Index : Integer) : Tcapteur;
begin
  Result := Tcapteur(inherited Get(Index))
end;

function TlisteCapteur.IndexOf(Item : Tcapteur) : Integer;
begin
  Result := inherited IndexOf(Item)
end;

procedure TlisteCapteur.Put(Index : Integer;Item : Tcapteur);
begin
  inherited Put(Index,Item)
end;

function TlisteCapteur.Remove(Item : Tcapteur) : Integer;
begin
  Result := inherited Remove(Item);
  Item.free;
end;

Procedure TListeCapteur.clear;
var i : Integer;
begin
  for i := 0 to pred(Count) do Items[i].Free;
  inherited Clear;
end;

constructor Tcapteur.create;
begin
             AvecPotarC := false;
             AmpliPhysiqueC := false;
             AlimentationC := false;
             LinearC := true;
             ExpressionC := '';
             Val0Vc := 0;
             Val5Vc := 100;
             nomC := 'T';
             NomUniteC := 'C';
             signifC := '';
             aideC := '';
             calibreC := c_5;
             numeroC := 6;
             TensionC := Positive;
end;

procedure Tcapteur.assign(Acapteur : Tcapteur);
begin
             AvecPotarC := Acapteur.AvecPotarC;
             AmpliPhysiqueC := Acapteur.AmpliPhysiqueC;
             AlimentationC := Acapteur.AlimentationC;
             LinearC := Acapteur.LinearC;
             ExpressionC := Acapteur.ExpressionC;
             Val0Vc := Acapteur.val0vc;
             Val5Vc := Acapteur.val5Vc;
             nomC := Acapteur.nomC;
             NomUniteC := Acapteur.NomUniteC;
             signifC := Acapteur.signifC;
             aideC := Acapteur.aideC;             
             calibreC := Acapteur.calibreC;
             numeroC := Acapteur.numeroC;
             TensionC := Acapteur.tensionC;
             MiniAff := Acapteur.miniAff;
             MaxiAff := Acapteur.maxiAff;
end;

Procedure Tvoie.Assign(source : Tpersistent);
var avoie : Tvoie;
begin
          inherited assign(source);
          avoie := Tvoie(source);
          Calibre := avoie.calibre;
          Valeur0V := avoie.valeur0V;
          Valeur5V := avoie.valeur5V;
          NumeroOrphy := avoie.numeroOrphy;
          Tension := avoie.tension;
          signif := avoie.signif;
          aide := avoie.aide;
          Mini := avoie.mini;
          Maxi := avoie.maxi;
          Etalonnage := Avoie.etalonnage;
          Expression := Avoie.expression;
          Couleur := Avoie.couleur;
          Motif := Avoie.motif;
          GroupBox := avoie.GroupBox;
          LabelVoie := avoie.labelVoie;
          exposantAff := avoie.exposantAff;
          exponent := avoie.exponent;
          active := avoie.active;
          nbreAvantVirgule := avoie.nbreAvantVirgule;
          ValeurCourante := avoie.ValeurCourante;
          valeurAff := avoie.valeurAff;
          Inverse := avoie.inverse;
          AvecPotar := avoie.AvecPotar;
          AmpliPhysique := avoie.AmpliPhysique;
          Alimentation := avoie.Alimentation;
          MaxiFFT := avoie.maxiFFT;
          FPente := avoie.Fpente;
          FpenteV := avoie.FpenteV;
          FdecalageV := avoie.FdecalageV;
          Fdecalage := avoie.Fdecalage;
end;

Procedure GetCodeGP305(f : double;var iSB,iSA : integer);
const marche = 16;
      sinus = 12;
begin
{
  GBF := 1 50 mHz 5 Hz
  GBF := 3 5 Hz 500 Hz
  GBF := 2 500 Hz 50 kHz
}
      if (f<0.05) or (f>50000) then begin
          iSB := 0;
          iSA := 0;
          exit;
      end;
      iSB := marche+sinus;
      if f<5
         then begin
            iSB := iSB+1;
            iSA := round((f-0.05)/5*255);
         end
         else if f<500
              then begin
                   iSB := iSB+3;
                   iSA := round((f-5)/500*255);
              end
              else begin
                     iSB := iSB+2;
                     iSA := round((f-500)/50000*255);
              end;
end;

function Tvoie.getExpression : string;
begin
    if (parser=nil)
       then result := ''
       else result := parser.expression
end;

procedure Tvoie.setExpression(Avalue : string);
begin
    if Avalue<>'' then begin
        if parser=nil then begin
           Parser := Tparser.create(nil);
           VoltParser := Parser.SetVariable('u',0);
        end;
        while pos('V',Avalue)>0 do
            Avalue[pos('V',Avalue)] := 'u';
        Parser.expression := Avalue;
    end;
end;

function nomEtatAcq(etat : TetatAcq) : string;
begin
     case etat of
          eaAttenteSynchro : result := trWaitSynchro;
          eaAttenteReveil : result := trWaitReveil;
          eaTemporelLent : result := trAcqLente;
          eaAttenteResul : result := trAcqEnCours;
          eaMajResul,eaFlush : result := '';
          eaAttenteResulMag : result := trAttenteMag;
          eaAttenteStopMag : result := trAttenteStopMag;
          eaTerminee : result := trAcqFinie;
     end;
end;

function nomSynchro(mode : TModeSynchro) : string;
begin
     case mode of
         S_front : result := trFront;
         S_seuil : result := trSeuil;
         S_relaxe : result := trRelaxe;
         S_clavier : result := trClavier; 
     end;
end;

function nomModeAcq(mode : TModeAcq) : String;
begin
     case mode of
         AcqClavier : result := trClavier;
         AcqTemps : result := trTemporel;
         AcqPoint : result := trPointparpoint;
         AcqMagnum : result := trMagnum;
         AcqGP305 : result := trGP305;
         AcqSA : result := trSortieAna;
     end;
end;

procedure Tvoie.affecteCapteur(capteurCourant : TCapteur);
begin with CapteurCourant do begin
            nom := nomC;
            nomUnite := nomUniteC;
            signif := signifC;
            aide := aideC;
            numeroOrphy := numeroC;
            Mini := MiniAff;
            Maxi := MaxiAff;
            AmpliPhysique := AmpliPhysiqueC;
            if AmpliPhysique then begin
                numeroOrphy := 2;
                afficheErreur(erAmpli,0);
            end;
            avecPotar := avecPotarC;
//            avecCapteur := true;
            valeur0V := val0VC;
            valeur5V := val5VC;
//            setMinMax;
            if linearC
               then expression := ''
               else expression := expressionC;
            isPhotocolor := genre=gPhotocolor;
    end;
    indexCapteurAuto := 0;
end;

Procedure Tvoie.ChercheCapteur(genre : TgenreCapteur);
var i : integer;
    capteurCourant : TCapteur;
    trouve : boolean;
begin
    trouve := false;
    capteurCourant := nil;
    i := 0;
    while (i<listeCapteur.count) and not trouve do begin
         CapteurCourant := listeCapteur.items[i];
         trouve := capteurCourant.genre=genre;
         inc(i);
    end;
    if not trouve then begin
       capteurCourant := Tcapteur.create;
       ListeCapteur.add(CapteurCourant);
       case genre of
            gPhase : begin
                  capteurCourant.val0VC:=0;
                  capteurCourant.val5VC:=180;
                  capteurCourant.NomC:='';
                  capteurCourant.NomUniteC:='';
                  capteurCourant.SignifC:='Phase';
                  capteurCourant.MaxiAff:=180;
                  capteurCourant.MiniAff:=-180;
                  capteurCourant.NumeroC:=9;
                  capteurCourant.AideC:='Phase de "jaune" / "vert"';
            end;
            gEntree : begin
                  capteurCourant.val0VC:=0;
                  capteurCourant.val5VC:=5;
                  capteurCourant.NomC:='Ve';
                  capteurCourant.NomUniteC:='V';
                  capteurCourant.SignifC:='Tension efficace d''entre';
                  capteurCourant.MaxiAff:=10;
                  capteurCourant.MiniAff:=0;
                  capteurCourant.NumeroC:=8;
                  capteurCourant.AideC:='fil vert prise H';
            end;
            gSortie : begin
                  capteurCourant.val0VC:=0;
                  capteurCourant.val5VC:=5;
                  capteurCourant.NomC:='Vs';
                  capteurCourant.NomUniteC:='V';
                  capteurCourant.SignifC:='Tension efficace de sortie';
                  capteurCourant.MaxiAff:=10;
                  capteurCourant.MiniAff:=0;
                  capteurCourant.NumeroC:=11;
                  capteurCourant.AideC:='fil jaune prise H';
            end;
       end;
    end;
    AffecteCapteur(CapteurCourant);
end;

var RegIni : TRegIniFile;

initialization
    RegIni := TRegIniFile.create(RegistreGTI);
    NomFichierCapteurs := regIni.readString('Capteurs','Fichier','');
    RegIni.free;
    if (NomFichierCapteurs='') or
       not FileExists(NomFichierCapteurs) then
       NomFichierCapteurs := changeFileExt(Application.exeName,'.CPT');
    ListeCapteur := TlisteCapteur.create;
    ChargeListeCapteurs

Finalization
    ListeCapteur.clear;
    ListeCapteur.free;
end.
