unit MagnumU;

interface

uses Windows,SysUtils,Maths;

Type
    TcorrectionMagnum = (cRaccordParabole,cMeilleureParabole);

const PeriodeMagnum = 0.0036;
      PeriodeMagnumE : double = PeriodeMagnum;
      ZeroMagnum = 32767;
      NbrePointMagnum : integer = 4;
      CorrectionMagnum : TcorrectionMagnum = cMeilleureParabole;
      ChuteMagnum : boolean = false;
      ChuteNegatif : boolean = false;
      deltatMinMagnum = 50e-6; { 130 km/h = 100 s }
      OscillationMagnum : boolean = false;
      SupprChute0 : boolean = true;

var
    puissA,puissB : vecteur;
    yA,yB : vecteur;

procedure CorrigeMagnum(var X,Y : vecteur;Yint : vecteurInt;var N : indiceVecteur);
{
  X = instants de dtection du passage du front
  Yint = positions renvoyes par GTI (integer)
  Y = valeur relle tenant compte du pas et de la correction
  N = nombre de points (on va de 0  N-1)
}

procedure CorrigeChute(var Y : vecteur;Yint : vecteurInt;N : indiceVecteur);

implementation

const
   degre = 2;
   ordre = degre+1;
   puissMax = 2*degre;

type
   matrice = array[1..ordre,1..ordre+1] of double;
   matriceLigne = array[1..ordre] of double;
   vecteurLoc = array[0..puissMax] of double;
   EsystemeLineaireError = class(Exception);

Procedure Resolution(A : matrice;var X : matriceLigne);
 { Cette procdure rsoud un systme AX = B }
 { La dernire colonne de A est en fait B }
 { par la mthode d'limination de Gauss (pivot partiel) }

  Procedure Elimination;

    FUNCTION Pivot(i : integer):integer;
    Var n,temp : integer;
    BEGIN
      temp := i;
      for n := i+1 to ordre do
          if abs(A[n,i]) > abs(A[i,i]) then temp:= n;
      pivot:=temp;
      if abs(A[temp,i]) <= 1e-100 then
         raise EsystemeLineaireError.Create('')
    END;

    PROCEDURE Permute(i,j : integer);
    Var temp : double;
        k : integer;
    BEGIN
       if i<>j then For k:=i to ordre+1 do BEGIN
          temp:=A[i,k];
          A[i,k]:=A[j,k];
          A[j,k]:=temp;
       END;
    END;

  var I,J,K : integer;
      M : double;
  begin
    for k := 1 to ordre do begin
          Permute(k,pivot(k));
          for i:=k+1 to ordre do begin
              M := A[i,k]/A[k,k];
              for j:=k+1 to ordre+1 do A[i,j]:=A[i,j]-M*A[k,j];
              A[i,k]:=0.0;
          end;
       end;
    end; {Elimination}

    Procedure Substitution;
    var I,J : integer;
        S : extended;
    begin
         for I:=ordre downto 1 do begin
             S:=A[I,ordre+1];
             for J:=I+1 to ordre do S:=S-A[I,J]*X[J];
             X[I]:=S/A[I,I];
         end;
    end;

begin
  Elimination;
  Substitution
end; {Resolution}

procedure CorrigeMagnum(var X,Y : vecteur;Yint : vecteurInt;var N : indiceVecteur);
{ On modlise au voisinage d'un extremum par une parabole  droite et
une parabole  gauche avec NbrePointMagnum de chaque ct.
On raccorde ensuite les deux paraboles en dcalant
soit d'un trait soit d'un pas et d'un trait
Le trait vient du fait que la dtection du front descendant ne se fait
pas du mme ct du trait selon le sens.
On suppose que le trait a comme taille la moiti d'un pas
Le pas vient du fait que l'on peut ou non dpasser le trait ce qui
cre une erreur alatoire d'un pas.
On en profite pour centrer }

var A,B : matrice;
    CoeffA,coeffB : matriceLigne;
    trait : double;
    isMaxi : boolean;

procedure CorrigeRaccord(ii : integer);
var i,j : integer;
    loc : double;
    xx,ecart : double;
    xA,xB : double; { maxi des paraboles }
{ A parabole avant, B parabole aprs }
begin
    puissA[0] := NbrePointMagnum;
    puissB[0] := NbrePointMagnum;
    for i := 1 to puissMax do begin
        puissA[i] := 0;
        puissB[i] := 0;
    end;
    for i := 1 to ordre do begin
       A[i,ordre+1] := 0;
       B[i,ordre+1] := 0;
    end;
    for j := succ(ii-NbrePointMagnum) to ii do begin
        xx := x[j]-x[ii];
        loc := xx;
        for i := 1 to puissMax do begin
            puissA[i] := puissA[i]+loc;
            loc := loc*xx;
        end;
        loc := y[j]-y[ii];
        for i := 1 to ordre do begin
            A[i,ordre+1] := A[i,ordre+1]+loc;
            loc := loc*xx;
        end;
    end;
    for j := succ(ii) to (ii+NbrePointMagnum) do begin
        xx := x[j]-x[ii];
        loc := xx;
        for i := 1 to puissMax do begin
            puissB[i] := puissB[i]+loc;
            loc := loc*xx;
        end;
        loc := y[j]-y[ii];
        for i := 1 to ordre do begin
            B[i,ordre+1] := B[i,ordre+1]+loc;
            loc := loc*xx;
        end;
    end;
    for i := 1 to ordre do
         for j := 1 to ordre do begin
             A[i,j] := PuissA[i+j-2];
             B[i,j] := PuissB[i+j-2];
         end;
    try
    Resolution(A,CoeffA);
    Resolution(B,CoeffB);
    xA := -coeffA[2]/2/coeffA[3]; { -b/2a }
    xB := -coeffB[2]/2/coeffB[3]; { -b/2a }
    xx := (xA+xB)/2; { xx milieu des sommets des paraboles }
    if (xx<0) or (xx>(x[ii+1]-x[ii])) then
        xx := (x[ii]+x[ii+1])/2-x[ii];
{ milieu entre monte et descente }        
    ecart := (coeffB[1]+xx*(coeffB[2]+xx*coeffB[3]))-
             (coeffA[1]+xx*(coeffA[2]+xx*coeffA[3]));
{ cart entre les sommets des paraboles }
    ecart := ecart/PeriodeMagnumE;
{ |ecart| devrait tre soit 0.5 soit 1.5 }
    if isMaxi
       then {Maxi} if ecart<-1
           then ecart := 3*PeriodeMagnumE/2
           else ecart := PeriodeMagnumE/2
       else {Mini} if ecart>1
           then ecart := -3*PeriodeMagnumE/2
           else ecart := -PeriodeMagnumE/2;
     for i := succ(ii) to pred(N) do
         Y[i] := Y[i]+ecart;
     except
     end;
end; { CorrigeRaccord }

procedure CorrigeBrut(ii : integer);
var i : integer;
    ecart : double;
begin
    if isMaxi
       then ecart := PeriodeMagnumE/2
       else ecart := -PeriodeMagnumE/2;
    for i := succ(ii) to pred(N) do Y[i] := Y[i]+ecart;
end; { CorrigeBrut }

procedure Corrige(ii : integer);
var i,j : integer;
    loc,locA,locB : double;
    xx,yy,ecartA,ecartB,ecart : double;
begin
    for i := 1 to puissMax do begin
        puissA[i] := 0;
        puissB[i] := 0;
    end;
    puissA[0] := 2*NbrePointMagnum;
    puissB[0] := 2*NbrePointMagnum;
    for i := 1 to ordre do begin
        A[i,ordre+1] := 0;
        B[i,ordre+1] := 0;
    end;
    for j := succ(ii-NbrePointMagnum) to ii do begin
        xx := x[j]-x[ii];
        loc := xx;
        for i := 1 to puissMax do begin
            puissA[i] := puissA[i]+loc;
            puissB[i] := puissA[i];
            loc := loc*xx;
        end;
        loc := y[j]-y[ii];
        yB[j] := loc;
        yA[j] := loc;
        for i := 1 to ordre do begin
            A[i,ordre+1] := A[i,ordre+1]+loc;
            B[i,ordre+1] := A[i,ordre+1];
            loc := loc*xx;
        end;
    end;
    for j := succ(ii) to ii+NbrePointMagnum do begin
        xx := x[j]-x[ii];
        loc := xx;
        for i := 1 to puissMax do begin
            puissA[i] := puissA[i]+loc;
            puissB[i] := puissA[i];
            loc := loc*xx;
        end;
        locA := y[j]-y[ii];
        if isMaxi
           then begin
           { effectue -1 au lieu de -1/2 donc correction A +1/2 }
           {                  ou de +1/2 donc correction B +1+1/2 }
              locA := locA+Trait;
              locB := locA+PeriodeMagnumE
           end
           else begin
           { effectue +1 au lieu de +1/2 donc correction A -1/2 }
           {                  ou de -1/2 donc correction B -1-1/2 }

              locA := locA-Trait;
              locB := locA-PeriodeMagnumE;
           end;
        yA[j] := locA;
        yB[j] := locB;
        for i := 1 to ordre do begin
            A[i,ordre+1] := A[i,ordre+1]+locA;
            B[i,ordre+1] := B[i,ordre+1]+locB;
            locA := locA*xx;
            locB := locB*xx;
        end;
    end;
    for i := 1 to ordre do
         for j := 1 to ordre do begin
             A[i,j] := PuissA[i+j-2];
             B[i,j] := PuissB[i+j-2];
         end;
    try
    Resolution(A,CoeffA);
    Resolution(B,CoeffB);
    ecartA := 0;
    ecartB := 0;
    for j := succ(ii-NbrePointMagnum) to ii+NbrePointMagnum do begin
        xx := x[j]-x[ii];
        ecartA := ecartA+sqr((coeffA[1]+xx*(coeffA[2]+xx*coeffA[3]))-yA[j]);
        ecartB := ecartB+sqr((coeffB[1]+xx*(coeffB[2]+xx*coeffB[3]))-yB[j]);
    end;
{ on garde la meilleure modif }
    if ecartA>ecartB
       then ecart := Trait+PeriodeMagnumE
       else ecart := Trait;
    if not isMaxi then ecart := -ecart;
    for i := succ(ii) to pred(N) do
         Y[i] := Y[i]+ecart;
    except
    end;
end; { Corrige }

var i,Nlimite : integer;
    maxi,mini,milieu : double;
    j,ecart : integer;
begin { CorrigeMagnum }
    trait := PeriodeMagnumE/2;
    NbrePointMagnum := 4;
    for i := 0 to pred(N) do
        Y[i] := Yint[i]*periodeMagnumE;
    if N<2*NbrePointMagnum then exit;
    j := 0;
    for i := 0 to pred(N) do
        if Yint[i]=Yint[i+1] then inc(j);
    if j>(N div 2) then begin // Problme EB5
        for i := 0 to pred(N) do
            Y[i] := i*periodeMagnumE;
        exit;
    end;
    try
    i := 1;
    repeat
         if (x[i]-x[i-1])<deltatMinMagnum
            then begin
              { suppression du point (i) aberrant et recalage de la suite }
                 if Yint[i]>Yint[i-1] then ecart := -1 else ecart := +1;
                 for j := i to pred(N) do begin
                     X[j] := X[j+1];
                     Yint[j] := Yint[j+1]+ecart;
                 end;
                 dec(N);
            end
            else inc(i);
    until (i>=N) or (N<=1);
    { recalcul en cas de correction }
    for i := 0 to pred(N) do Y[i] := Yint[i]*periodeMagnumE;
    i := NbrePointMagnum+1;
    if oscillationMagnum then begin
      repeat
         if (Yint[i]-Yint[i-1])*(Yint[i+1]-Yint[i])<0 then begin
           { Changement de sens }
              isMaxi := yint[i+1]<yint[i];
           { Recherche amplitude }
              j := i+1;
              while (j<N) and
                    ((Yint[j]-Yint[j-1])*(Yint[j+1]-Yint[j])>0) do inc(j);
              j := j-i; 
              if j<3 then j := 3;
              NbrePointMagnum := j div 3;
        { sinusode assimile  une parabole sur 1/3 de priode }
              if NbrePointMagnum>5 then NbrePointMagnum := 5;
              if NbrePointMagnum<3 then NbrePointMagnum := j div 2;
        { sinusode assimile  une parabole sur 1/2 priode }              
         { correction }
              if NbrePointMagnum<3 { minimum pour une parabole ! }
                 then CorrigeBrut(i)
                 else case correctionMagnum of
                      cMeilleureParabole : Corrige(i);
                      cRaccordParabole : CorrigeRaccord(i);
                 end;
              inc(i,j-2);
         end;
         inc(i);
      until i>=(N-NbrePointMagnum-1);
      GetMinMax(Y,N,mini,maxi);
      Milieu := (maxi+mini)/2;
      for i := 0 to pred(N) do Y[i] := Y[i]-milieu
    end { oscillation }
    else begin
         Nlimite := N-NbrePointMagnum;
         repeat inc(i)
         until ((Yint[i]-Yint[i-1])*(Yint[i+1]-Yint[i])<0) or
               (i>Nlimite);
         if i<Nlimite then { parabole }
             case correctionMagnum of
                cMeilleureParabole : Corrige(i);
                cRaccordParabole : CorrigeRaccord(i);
             end;
    end; 
    { else tester 1er et dernier point ? }
    except
    end;
end; { CorrigeMagnum }

procedure CorrigeChute(var Y : vecteur;Yint : vecteurInt;N : indiceVecteur);
var i : integer;
    trait : double;
begin
    trait := periodeMagnumE;
    if ChuteNegatif then trait := -trait;
    for i := 0 to pred(N) do Y[i] := Yint[i]*trait;
end; { CorrigeChute }

end.
